/*
 * QR Code generator library (Rust)
 *
 * Copyright (c) Kanashius (https://www.kanashius.com)
 */

extern crate qrcodegen;

use std::env;
use std::process::exit;
use qrcodegen::{QrCode, QrCodeEcc};

fn main() {
    let mut errcorlvl: Option<QrCodeEcc> = None;
    let mut text: Option<String> = None;
    let mut human: bool = false;
    let args: Vec<String> = env::args().collect();
    let mut i = 1;
    while i<args.len(){
        match args.get(i) {
            None => {}
            Some(x) if x == "--errorlvl" => { // --errorlvl low 7%/medium 15%/quartile 25%/high 30%
                errcorlvl = Some(match args.get(i+1) {
                    None => {
                        println!("No value for --errorlvl defined and the text argument is missing!\n");
                        print_help();
                        std::process::exit(1);
                    }
                    Some(x) => {
                        i+=1;
                        match x {
                            _ if x.eq_ignore_ascii_case("low") => {QrCodeEcc::Low}
                            _ if x.eq_ignore_ascii_case("medium") => {QrCodeEcc::Medium}
                            _ if x.eq_ignore_ascii_case("quartile") => {QrCodeEcc::Quartile}
                            _ if x.eq_ignore_ascii_case("high") => {QrCodeEcc::High}
                            _ => {
                                println!("Value for --errorlvl is not allowed: {}\n", x);
                                print_help();
                                std::process::exit(1);
                            }
                        }
                    }
                })
            }
            Some(x) if x == "--help" => {
                print_help();
                std::process::exit(0);
            }
            Some(x) if x == "--human" => {
                human = true;
            }
            Some(val) => {
                text = Some(val.clone());
            }
        }
        i+=1;
    }
    if text.is_none() {
        println!("No text was provided!\n");
        print_help();
        std::process::exit(1);
    }
    match QrCode::encode_text(&text.unwrap(), errcorlvl.unwrap_or(QrCodeEcc::High)) {
        Ok(qr) => {
            if human {
                print_qr_human(&qr);
            }else{
                print_qr(&qr);
            }
        }
        Err(e) => {
            println!("Error running qrcode.exe: {0}", e);
            exit(2)
        }
    };
}

fn print_qr_human(qr: &QrCode) {
    let border: i32 = 0;
    for y in -border .. qr.size() + border {
        for x in -border .. qr.size() + border {
            let c: char = if qr.get_module(x, y) { '█' } else { ' ' };
            print!("{0}{0}", c);
        }
        println!();
    }
    println!();
}

fn print_qr(qr: &QrCode) {
    let border: i32 = 0;
    for y in -border .. qr.size() + border {
        for x in -border .. qr.size() + border {
            let c: char = if qr.get_module(x, y) { '1' } else { '0' };
            print!("{0}", c);
        }
        if y < qr.size() + border-1{ // no 2 and the end
            print!("2");
        }
    }
}

fn print_help(){
    println!("QRCode generator based on the QR-Code-Generator (see License below) \
              and wrapped/modified by Kanashius");
    println!();
    println!("qrcode <options> text");
    println!();
    println!("Options:");
    println!("--help\t\t\tShow the help");
    println!("--human\t\t\tMake the QRCode visible to the eye. Otherwise it is encoded as \
                0-background, 1-fill, 2-linebreak");
    println!("--errorlvl <value>\tDefine the amount of possible error correction: \
                                    low (7%) medium (15%) quartile (25%) or high (30%). \
                                    DEFAULT: high");
    println!();
    println!("-------------------------------------------");
    println!("License (original library)");
    println!("Copyright © 2024 Project Nayuki. (MIT License)");
    println!("https://www.nayuki.io/page/qr-code-generator-library");
    println!("Permission is hereby granted, free of charge, to any person obtaining a copy of this \
              software and associated documentation files (the \"Software\"), to deal in the \
              Software without restriction, including without limitation the rights to use, copy, \
              modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, \
              and to permit persons to whom the Software is furnished to do so, subject to the \
              following conditions:");
    println!("\t- The above copyright notice and this permission notice shall be included in all \
              copies or substantial portions of the Software.");
    println!("\t- The Software is provided \"as is\", without warranty of any kind, express or \
              implied, including but not limited to the warranties of merchantability, fitness for \
              a particular purpose and noninfringement. In no event shall the authors or copyright \
              holders be liable for any claim, damages or other liability, whether in an action of \
              contract, tort or otherwise, arising from, out of or in connection with the Software \
              or the use or other dealings in the Software.");
}